MODULE transport_unstructured_mod
  USE ISO_C_BINDING
  USE OMP_LIB
  USE data_unstructured_mod
  IMPLICIT NONE
  PRIVATE
  SAVE

#define BINDC_(thename) BIND(C, name=#thename)
#define BINDC(thename) BINDC_(dynamico_ ## thename)

#define DBL REAL(C_DOUBLE)
#define DOUBLE1(m) DBL, DIMENSION(m)
#define DOUBLE2(m,n) DBL, DIMENSION(m,n)
#define DOUBLE3(m,n,p) DBL, DIMENSION(m,n,p)
#define DOUBLE4(m,n,p,q) DBL, DIMENSION(m,n,p,q)
#define INDEX INTEGER(C_INT)

CONTAINS

#define FIELD_PS      DOUBLE1(primal_num)
#define FIELD_MASS    DOUBLE2(llm, primal_num)
#define FIELD_Z       DOUBLE2(llm, dual_num)
#define FIELD_U       DOUBLE2(llm, edge_num)
#define FIELD_UL      DOUBLE2(llm+1, edge_num)
#define FIELD_THETA   DOUBLE3(llm, primal_num, nqdyn)
#define FIELD_GEOPOT  DOUBLE2(llm+1, primal_num)

#define HASNAN(field) (ANY(.NOT.ABS(field)<1e20))

  SUBROUTINE remap_unstructured(rhodz, thetarhodz, u) BINDC(remap)
    FIELD_MASS   :: rhodz, thetarhodz    ! INOUT
    FIELD_U      :: u                    ! INOUT
    ! DIAG
    FIELD_PS     :: new_rhodz_cum, mass_col
    FIELD_MASS   :: old_rhodz
    FIELD_GEOPOT :: eta, rhodz_cum, thetarhodz_cum, new_thetarhodz_cum
    FIELD_U      :: urhodz
    FIELD_UL     :: urhodz_cum,  new_urhodz_cum
    INTEGER, DIMENSION(primal_num) :: cur_lev

    DBL          :: rhodz_cum_target, rhodz_cum_lev, rhodz_cum_levp1, X, time1,time2
    INTEGER :: ij, l, level, edge, ij_left, ij_right
    
    time1=OMP_GET_WTIME()
    
#include "../kernels_unst/remap_eta.k90"
#include "../kernels_unst/remap_theta.k90"
#include "../kernels_unst/remap_u.k90"
    
    time2=OMP_GET_WTIME()
    !  CALL CPU_TIME(time2)
    IF(time2>time1) elapsed = elapsed + time2-time1

    CALL print_trace()
  END SUBROUTINE remap_unstructured

END MODULE transport_unstructured_mod
